Import BRL.Max2D
Import Pub.Win32

' -----------------------------------------------------------------------------
' How to use...
' -----------------------------------------------------------------------------

' -----------------------------------------------------------------------------
' 1) Call OpenCaptureDevice with device number (0 onwards) and required image
' size), using TCaptureDevice handle for return value:
' -----------------------------------------------------------------------------

' 		cap:TCaptureDevice = OpenCaptureDevice (device, width, height)

' -----------------------------------------------------------------------------
' 2) Obtain device name by passing device number (no open device needed)...
' -----------------------------------------------------------------------------

' 		Print CaptureDeviceName (id)

' -----------------------------------------------------------------------------
' 3) Capture a frame from device by passing TCaptureDevice handle and using
' TImage handle to store result (should be checked for Null value)...
' -----------------------------------------------------------------------------

'		image:TImage = CaptureFrame (cap)

'
'		DrawImage image, 0, 0
'		' Etc...

' -----------------------------------------------------------------------------
' 4) Close device on exiting program by passing TCaptureDevice handle...
' -----------------------------------------------------------------------------

' 		CloseCaptureDevice (cap)

' -----------------------------------------------------------------------------

' -----------------------------------------------------------------------------
' ESCAPI wrapper stuff...
' -----------------------------------------------------------------------------

Type SimpleCapParams
	Field mTargetBuf:Byte Ptr ' Must be at least mWidth * mHeight * SizeOf(Int) of size! 
	Field mWidth
	Field mHeight
End Type

Global InitCOM () "C"
Global CountCaptureDevices () "C"
Global InitCapDevice (device, scp:Byte Ptr) "C"
Global CloseCapDevice (device) "C"
Global GetCap (device) "C"
Global CapDone (device) "C"
Global CapDevice (device, name:Byte Ptr, namelen) "C"
Global ESCAPIDLLVersion () "C"

Function SetupESCAPI ()

	esc = LoadLibraryA ("escapi.dll")
	
	If esc
	
		InitCOM				= GetProcAddress (esc, "initCOM")
		CountCaptureDevices	= GetProcAddress (esc, "countCaptureDevices")
		InitCapDevice 		= GetProcAddress (esc, "initCapture")
		CloseCapDevice		= GetProcAddress (esc, "deinitCapture")
		GetCap				= GetProcAddress (esc, "doCapture")
		CapDone				= GetProcAddress (esc, "isCaptureDone")
		CapDevice			= GetProcAddress (esc, "getCaptureDeviceName")
		ESCAPIDLLVersion	= GetProcAddress (esc, "ESCAPIDLLVersion")

		If InitCOM = Null Or CountCaptureDevices = Null Or InitCapDevice = Null Or..
			CloseCapDevice = Null Or GetCap = Null Or CapDone = Null Or..
				CapDevice = Null Or ESCAPIDLLVersion = Null
					DebugLog "Function missing!"
					Return 0
		EndIf

		InitCOM ()
		
		If ESCAPIDLLVersion () < $200
			DebugLog "Old DLL (needs version 2.0+)"
			Return 0
		EndIf
		
	Else
		DebugLog "Failed to open DLL"
		Return 0
	EndIf

	Return 1
	
End Function

Function CaptureDeviceName$ (device)
	Local devname:Byte [1024]
	CapDevice (device, devname, 1024)
	Return String.FromCString (devname)
End Function

Type TCaptureDevice
	Field scp:SimpleCapParams
	Field pix:TPixmap
	Field image:TImage
	Field device:Int
End Type

Function OpenCaptureDevice:TCaptureDevice (device, width, height)

	capdev:TCaptureDevice = New TCaptureDevice

	capdev.device = device	
	capdev.pix = CreatePixmap (width, height, PF_BGRA8888)
	capdev.image = CreateImage (width, height)
	
	capdev.scp = New SimpleCapParams
	capdev.scp.mTargetBuf = PixmapPixelPtr (capdev.pix)
	capdev.scp.mWidth = width
	capdev.scp.mHeight = height
	
	If Not InitCapDevice (capdev.device, capdev.scp)
		capdev = Null
	EndIf
	
	Return capdev
	
End Function

Function CloseCaptureDevice:TCaptureDevice (capdev:TCaptureDevice)
	CloseCapDevice capdev.device
	capdev.pix		= Null
	capdev.image	= Null
	capdev.scp		= Null
	Return Null
End Function

Function CaptureFrame:TImage (capdev:TCaptureDevice, timeout = 1000)

	GetCap capdev.device

	timeout_start = MilliSecs ()

	Repeat
		Delay 1
		If MilliSecs () > timeout_start + timeout
			DebugLog "Timed out waiting for frame! Device unplugged?"
			capdev.image = Null
			Exit
		EndIf
	Until CapDone (capdev.device)

	If capdev.image
	
		' Draw Pixmap to back buffer and grab into image...
	
		DrawPixmap capdev.pix, 0, 0
		GrabImage (capdev.image, 0, 0)
		
	EndIf
	
	Return capdev.image

End Function

If SetupESCAPI () = 0
	Notify "Error! Make sure escapi.dll is in program folder!"
	End
EndIf
